#!/usr/bin/perl

use strict;
use warnings;

use Time::Local;
use Getopt::Std;
$Getopt::Std::STANDARD_HELP_VERSION = 1;

my $VERSION = "0.1";

my (%opts, $prog, $status_count, $arg_ip, $arg_sort, $arg_unique, $arg_filter, $filter_active, $filter_backup, $filter_free);
$status_count 	= 0;
$arg_ip 		= 0;
$arg_unique 	= 0;
$filter_active 	= 0;
$filter_backup 	= 0;
$filter_free	= 0;
$prog = $0;

sub VERSION_MESSAGE {
	print $prog . " v." . $VERSION;
	exit(0);
}

sub HELP_MESSAGE {
my $usage = <<END;
NAME
getdhcp - List dhcp leases assigned on dhcp server.

SYNOPSIS
    getdhcp -f [a,b,f]
    getdhcp -i
    getdhcp -s [1-7]
    getdhcp -u

DESCRIPTION
    This program looks up the assigned dhcp leases on the server and displays information about them.

    The following options are available:	
    -f [a,b,f]          Filter output.
                        a = only display records with an active lease status.
                        b = only display records with a backup lease status.
                        f = only display records with a free lease status.

    -h                  Usage

    -i                  Only display the ip address in the output.

    -s [1-7]            Sort order of output.
                        Number corresponds to the output column to use for sorting, from left to right respectively.

                        1 = Lease start
                        2 = Lease stop
                        3 = State - current
                        4 = State - next
                        5 = IP Address [default]
                        6 = MAC Address
                        7 = Client Name

    -u                  Unique. Only display unique ip address records.
                        This option always displays the last record found for that ip address.	


EXAMPLES
steve\@ns-11: getdhcp -i -f a
IP ADDRESSES (active)
========================q
192.168.8.76
192.168.8.77
192.168.8.79
192.168.8.80
192.168.8.81
192.168.8.82
...

steve\@ns-12: getdhcp -s 5 -u
LEASE STARTS               LEASE STOPS                STATE_CURR  STATE_NEXT  IP ADDRESS        MAC ADDRESS         CLIENT NAME
========================   ========================   ==========  ==========  ================  =================   ===============
Tue Jul 30 16:29:18 2013   Tue Jul 30 18:29:18 2013   active      expired     192.168.8.76      78:a3:e4:31:f6:fc   Michelle
Tue Jul 30 16:43:35 2013   Tue Jul 30 18:43:35 2013   active      expired     192.168.8.77      4c:b1:99:cd:90:90
...

END

	print $usage;
	exit(0);
}

my $opts_status = getopts('f:s:hiu', \%opts);
if (!$opts_status) {
	print "\n";
	HELP_MESSAGE();
}

if (defined($opts{f})) {
	$arg_filter = $opts{f};
	if ($arg_filter !~ /(?:a|b|f)$/) {
		print "\nError: incorrect filter option specified! [a, b or f required]\n\n";
		exit(1);
	}
	if ($arg_filter =~ /a/) {$filter_active = 1;}
	if ($arg_filter =~ /b/) {$filter_backup = 1;}
	if ($arg_filter =~ /f/) {$filter_free = 1;}
	$status_count++;
}

if (defined($opts{h})) {
	HELP_MESSAGE();
}

if (defined($opts{i})) {
	$arg_ip = 1;
}

if (defined($opts{s})) {
	$arg_sort = $opts{s};
	if ($arg_sort !~ /^[1-7]$/) {
		print "\nError: incorrect sort option specified! [1-7 required]\n\n";
		exit(1);
	}
} else {
	$arg_sort = 5;
}

if (defined($opts{u})) {
	$arg_unique = 1;
}

if ($status_count > 1) {
	print "\nError: multiple status options not allowed!\n\n";
	exit(1);
}

my $now = time();

my ($data, $ip, $date, $time, $y, $m, $d, $H, $M, $S, $start, $stop, $state_curr, $state_next, $mac, $client, 
    $output, @records, $track_record, %ip_count, $store_record_key, %store_records, %track_ips);

select (STDOUT);
$~ = "OUTPUT";

format OUTPUT =
@<<<<<<<<<<<<<<<<<<<<<<<   @<<<<<<<<<<<<<<<<<<<<<<<   @<<<<<<<<<  @<<<<<<<<<  @<<<<<<<<<<<<<<<  @<<<<<<<<<<<<<<<<   @*
$start, $stop, $state_curr, $state_next, $ip, $mac, $client
.

my $lease_file = "/var/db/dhcpd/dhcpd.leases";
#my $lease_file = "./dhcpd.leases.never";
open(LEASE, "$lease_file") or die ("Could not open $lease_file file.");;
while (my $line = <LEASE>) {
	chomp ($line);
	if ($line !~ /^lease /) {next};
	$ip = ""; $start = ""; $stop = ""; $mac = ""; $client = ""; $state_curr = ""; $state_next = "";
	until ($line =~ /^}/) {
		if ($line =~ /^lease /) {
			$ip = (split(" ", $line))[1];
			$ip_count{$ip}++;
		} elsif ($line =~ /^  starts/) {
			($date, $time) = (split(" ", $line))[2,3];
			($y, $m, $d) = split("/", $date);
			($H, $M, $S) = split(":", $time);
			$start = timelocal($S,$M,$H,$d,$m-1,$y);
		} elsif ($line =~ /^  ends never/) {
			$stop = "never";
		} elsif ($line =~ /^  ends/) {
			($date, $time) = (split(" ", $line))[2,3];
			($y, $m, $d) = split("/", $date);
			($H, $M, $S) = split(":", $time);
			$stop = timelocal($S,$M,$H,$d,$m-1,$y);
		} elsif ($line =~ /^  next binding state/) {
			$state_next = (split(" ", $line))[3];
			$state_next =~ s/;//;
		} elsif ($line =~ /^  binding state/) {
			$state_curr = (split(" ", $line))[2];
			$state_curr =~ s/;//;
		} elsif ($line =~ /^  hardware ethernet/) {
			$mac = (split(" ", $line))[2];
			$mac =~ s/;//;
		} elsif ($line =~ /^  client-hostname/) {
			$client = (split(/\"/, $line))[1];
		}
		$line = <LEASE>;
		chomp ($line);
	}
	my $record = $start.",".$stop.",".$state_curr.",".$state_next.",".$ip.",".$mac.",".$client;
	if ($arg_unique == 1) {
		$store_record_key = $ip . sprintf("%0.5d", $ip_count{$ip});
		$store_record_key =~s /\.//g;
		$store_records{$store_record_key} = $record;
	} else {
		push (@records, $record);
	}

	#print localtime($start) . "\t" . localtime($stop) . "\t$state_curr\t$state_next\t\t$ip\t$mac\t$client\n" if $stop >= $now;
	#print localtime($start) . "\t" . localtime($stop) . "\t$state_curr\t\t$state_next\t\t$ip\t$mac\t$client\n";
	#write();
}
close(LEASE);

if ($arg_unique == 1) {
	foreach my $key (reverse sort keys %store_records) {
		my $record = $store_records{$key};
		($start, $stop, $state_curr, $state_next, $ip, $mac, $client) = split(/,/, $record, 7);
		$track_ips{$ip}++;
		if (defined($track_ips{$ip}) && $track_ips{$ip} == 1) {
			push (@records, $record);	
		}
	}
}

my @records_sorted = sort {
	if ($arg_sort > 0) {
		(split ',', $a, 7)[$arg_sort-1] cmp (split ',', $b, 7)[$arg_sort-1]
	}
  } @records;

my $record_count = 0;
foreach my $record (@records_sorted) {
	$record_count++;
	$stop = "";
	if ($record_count == 1) {header();}
	($start, $stop, $state_curr, $state_next, $ip, $mac, $client) = split(/,/, $record, 7);
	if ($filter_active == 1 || $filter_backup == 1 || $filter_free == 1) {
		if (($filter_active == 1 && $state_curr eq "active") ||
			($filter_backup == 1 && $state_curr eq "backup") ||
			($filter_free == 1 && $state_curr eq "free")) {# do nothing}
		} else {next;}
	}

	if ($arg_ip == 1) {
		print "$ip\n";
	} else {
		if ($arg_unique == 1 && defined($track_record->{$record})) {
			next;
		}

		$track_record->{$record} = 1;
		$start = localtime($start);
		if (defined($stop) && $stop ne "" && $stop ne "never") {
			$stop = localtime($stop);
		} 
		write();
	}
}

sub header {
	my $state;
	if ($arg_ip == 1) { 			
		if ($filter_active == 1)		{$state = " (active)";}
		elsif ($filter_backup == 1) 	{$state = " (backup)";}
		elsif ($filter_free == 1)		{$state = " (free)";}
		else {$state = "";}
		print "IP ADDRESSES${state}\n";
		print "========================\n";
	} else {
		$start 		= "LEASE STARTS";
		$stop 		= "LEASE STOPS";
		$state_curr	= "STATE_CURR";
		$state_next	= "STATE_NEXT";
		$ip 		= "IP ADDRESS";
		$mac 		= "MAC ADDRESS";
		$client 	= "CLIENT NAME";
		write();

		print "========================   ========================   ==========  ==========  ================  =================   ===============\n";
	}
}

exit;
